<?php

namespace App\Http\Controllers\API;

use App\Http\Controllers\Controller;
use App\Http\Requests\API\File\DownloadZipRequest;
use App\Http\Requests\API\File\FileUploadRequest;
use App\Http\Requests\API\File\KeepDirectoryUploadRequest;
use App\Http\Resources\API\BimFileResource;
use App\Http\Resources\API\FileByObjectResource;
use App\Http\Resources\API\FileDownloadResource;
use App\Http\Resources\API\FileUploadSuccessResource;
use App\Libraries\BIM\BIMFactory;
use App\Libraries\ChunkedUpload\Disks\Cos;
use App\Models\BimFile;
use App\Models\Container;
use App\Models\Enums\BimFileModelType;
use App\Models\Enums\FileObjectType;
use App\Models\Enums\ObjectAction;
use App\Models\File;
use App\Repositories\ActionRepository;
use App\Services\File\DownloadService;
use App\Services\File\FileHideService;
use App\Services\File\Upload\CompanyUploadService;
use App\Services\File\Upload\FilesUploadService;
use App\Services\File\Upload\KeepDirectoryUploadService;
use App\Services\File\Upload\ProgressBar;
use Illuminate\Http\Request;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Storage;

class FileController extends Controller
{

    public function download(string $id)
    {
        $file = File::query()->findOrFail($id);

        $fileObjectType = FileObjectType::from($file->object_type);
        $object = $fileObjectType->modelBuilderAllowed($file->object_id)->find($file->object_id);
        if (!$object) {
            return $this->badRequest(sprintf("File ID: %s, no permission to access", $file->id));
        }

        ActionRepository::createByFile($file->object_id, $file->object_type, ObjectAction::DOWNLOAD);
        return new FileDownloadResource($file);
    }

    public function changeName(Request $request, string $id)
    {
        $isAction = $request->input('is_action', null);
        $result = ($isAction === '0') ? true : ($isAction === '1' ? false : false);

        $file = File::query()->findOrFail($id);

        $fileObjectType = FileObjectType::from($file->object_type);
        $object = $fileObjectType->modelBuilderAllowed($file->object_id, $result)->find($file->object_id);
        if (!$object) {
            return $this->badRequest(sprintf("File ID: %s, no permission to access", $file->id));
        }

        File::query()->where('title', $file->title)
            ->where('object_type', $file->object_type)
            ->where("object_id", $file->object_id)
            ->update(['title' => $request->get('title')]);

        ActionRepository::createByFile($file->object_id, $file->object_type, ObjectAction::EDITED_FILE);

        return $this->noContent();
    }

    public function destroy(Request $request, string $id)
    {
        $isAction = $request->input('is_action', null);
        $result = ($isAction === '0') ? true : ($isAction === '1' ? false : false);

        $file = File::query()->findOrFail($id);

        $fileObjectType = FileObjectType::from($file->object_type);
        $object = $fileObjectType->modelBuilderAllowed($file->object_id, $result)->find($file->object_id);
        if (!$object) {
            return $this->badRequest(sprintf("File ID: %s, no permission to access", $file->id));
        }

        $files = File::query()
            ->where('title', $file->title)
            ->where('object_type', $file->object_type)
            ->where("object_id", $file->object_id)
            ->get();

        File::query()->whereIn("id", $files->pluck("id")->toArray())->delete();

        ActionRepository::createByFile($file->object_id, $file->object_type, ObjectAction::DELETED_FILE);

        Storage::delete($files->pluck("pathname")->toArray());

        $filesSize = $files->sum("size");

        $company = Auth::user()->company;
        $company->decrement("used_storage_capacity", $filesSize);
        $company->save();

        return $this->noContent();
    }

    public function downloadZip(DownloadZipRequest $request, DownloadService $service)
    {
        return $this->success([
            'data' => $service->downloadByIds($request->get("ids")),
        ]);
    }

    public function downloadAllLatest(DownloadService $service, string $objectType, string $objectId)
    {
        return $this->success([
            'data' => $service->downloadAllLatest($objectType, $objectId),
        ]);
    }

    public function downloadShareFile(DownloadService $service, string $uuid)
    {
        return $this->success([
            'data' => $service->downloadShareFile($uuid),
        ]);
    }

    /**
     * 文件上传
     *
     * @param FileUploadRequest $request
     * @return FileUploadSuccessResource|\Illuminate\Http\JsonResponse
     */
    public function upload(FileUploadRequest $request)
    {
        set_time_limit(0);

        $uploadedFiles = (new FilesUploadService($request))->upload();

        if ($request->object_id !== null) {
            ActionRepository::createByFile($request->object_id, $request->object_type, ObjectAction::UPLOADED);
        }
        return $this->success([
            'data' => $uploadedFiles
        ]);

    }

    /**
     * 模型上传
     * @param \App\Http\Requests\API\File\FileUploadRequest $request
     * @return
     */
    public function modelUpload(FileUploadRequest $request)
    {
        set_time_limit(0);

        $uploadedFiles = (new FilesUploadService($request))->uploadModels();

        if ($request->object_id !== null) {
            ActionRepository::createByFile($request->object_id, $request->object_type, ObjectAction::UPLOADED);
        }

        return FileUploadSuccessResource::collection($uploadedFiles);
    }

    public function companyUpload(FileUploadRequest $request)
    {
        set_time_limit(0);
        $uploadedFiles = (new CompanyUploadService($request))->upload();

        return $this->success([
            'data' => $uploadedFiles
        ]);
    }


    /**
     * 文件夹上传
     *
     * @param KeepDirectoryUploadRequest $request
     * @return FileUploadSuccessResource|\Illuminate\Http\JsonResponse
     */
    public function keepDirectoryUpload(KeepDirectoryUploadRequest $request)
    {
        set_time_limit(0);

        $uploadedFiles = (new KeepDirectoryUploadService($request))->upload();

        if ($request->object_id !== null) {
            ActionRepository::createByFile($request->object_id, $request->object_type, ObjectAction::UPLOADED);
        }
        return $this->success([
            'data' => $uploadedFiles
        ]);
    }


    public function uploadProgress(string $requestId)
    {
        return $this->success([
            'data' => ProgressBar::queryByRequestId($requestId),
        ]);
    }

    public function byObject(string $objectType, string $objectId)
    {
        $fileObjectType = FileObjectType::from($objectType);
        $fileObjectType->modelBuilderAllowed($objectId)->findOrFail($objectId);

        $files = File::query()
            ->with(['createdBy'])
            ->where('object_type', $objectType)
            ->where('object_id', $objectId)
            ->orderByDesc("version")
            ->where("source", 1)
            ->get();


        $items = [];
        foreach ($files->groupBy("title") as $fileItems) {
            $item = (new FileByObjectResource($fileItems->first()))->toArray(request());
            $item['extraField'] = 'parent' . '_' . $item['id'];
            $item['children'] = [];

            foreach ($fileItems as $fileItem) {
                $resource = (new FileByObjectResource($fileItem))->toArray(request());
                $resource['extraField'] = 'children' . '_' . $resource['id'];
                $item['children'][] = $resource;
            }

            $items[] = $item;
        }

        return $this->success([
            'data' => $items
        ]);
    }

    /**
     * 文件隐藏
     *
     * @param string $id
     * @param FileHideService $service
     * @return \Illuminate\Http\Response
     */
    public function hide(string $id, FileHideService $service)
    {
        $service->hide($id);

        return $this->noContent();
    }

    public function bimView(string $id)
    {
        $file = File::query()->where('is_bim', 1)->findOrFail($id);

        $fileObjectType = FileObjectType::from($file->object_type);
        $object = $fileObjectType->modelBuilderAllowed($file->object_id)->find($file->object_id);
        if (!$object) {
            return $this->badRequest(sprintf("File ID: %s, no permission to access", $file->id));
        }

        $bimFile = $file->bimFile;
        if ($bimFile->convert_status === 5) {
            $result = BIMFactory::make($bimFile->bim_driver)->getModelDetail($file->bimFile);

            $stationUrl = config('bim.glendale.host');
            $result = Arr::only($result, ['modelAccessAddress', 'lightweightName', 'fileOwnership', 'floorJsonURL', 'fileType', 'initiatingUser', 'modelType', 'name']);
            if (isset($result['modelAccessAddress'])) {
                $result['modelAccessAddress'] = str_replace('http://159.75.168.101:18086', $stationUrl, $result['modelAccessAddress']);
            }

            // $result['baseUrl'] = config('bim.glendale.base_url');
            $result['stationUrl'] = $stationUrl;
            $result['stationToken'] = config('bim.glendale.token');
            $result['modelType'] = $bimFile->model_type;


            return $this->success([
                'data' => [
                    'bim_driver' => $bimFile->bim_driver,
                    'bim_view' => $result,
                ]
            ]);
        } else {
            return $this->badRequest("BIM is in the process of conversion. Please wait a moment");
        }

    }

    /**
     * 获取上传COS临时密钥
     */
    public function getCOSToken(FileUploadRequest $request, Cos $cos)
    {
        $pathPrefix = cos_upload_prefix(Auth::user()->company_id, $request['object_type'] ?? '');

        $data = $cos->getTempKeys($pathPrefix);

        return $this->successData($data);
    }

    /**
     * 容器下的所有模型列表
     */
    public function models(Request $request)
    {
        $containerId = $request->container_id;
        $pageSize = (int) $request->get('page_size', 10);
        $modelType = $request->get('model_type', 'bim');
        switch ($modelType) {
            case 'gis':
                $modelTypeArr = [BimFileModelType::GIS->value];
                break;
            default:
                $modelTypeArr = [BimFileModelType::BIM->value, BimFileModelType::CAD->value];
                break;
        }

        $container = Container::query()->allowed()->findOrFail($containerId);

        $bimFiles = BimFile::query()
            ->modelType($modelTypeArr)
            ->whereHas('file', function ($query) use ($container) {
                return $query->containerLibraryId($container->library_id);
            })
            ->with('file')
            ->latest('id')
            ->paginate($pageSize);

        return BimFileResource::collection($bimFiles);
    }
}
